# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for managing locations in Weather app"""

from __future__ import absolute_import

from unittest import skip
from testtools.matchers import Equals, Is, Not
from autopilot.matchers import Eventually
import logging

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin, SheetMixin

logger = logging.getLogger(__name__)


class TestSettings(WeatherTestCase, DatabaseMixin, SheetMixin):
    def setUp(self):
        #we want to start with fake settings data
        self.create_blank_db()
        logger.debug("Adding fake data to new database")
        self.add_locations_to_database()
        self.add_units_to_database()
        super(TestSettings, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def _open_settings_sheet(self):
        """Opens the settings sheet"""
        self.main_view.open_toolbar().click_button("SettingsButton")

    def _move_pointer_around(self):
        """
        Helper method to move the pointer around, to assure selector is opened
        """
        self.assertThat(lambda: self.main_view.select_single(
            'ComposerSheet', objectName='SettingsSheet'),
            Eventually(Not(Is(None))))
        sheet = self.main_view.select_single('ComposerSheet',
                                             objectName='SettingsSheet')
        self.pointing_device.move_to_object(sheet)

    def _check_units(self, units):
        """Checks selected units by values from the first location tab"""
        if self.main_view.wideAspect:
            self._ensure_tablet_tabs_loaded()
        self.assertThat(lambda: self.main_view.select_many(
            'QQuickText', objectName='CurrentTempText'),
            Eventually(Not(Is(None))))
        current_location_tab = self.main_view.wait_select_single(
            'LocationTab', visible=True)
        today_item = current_location_tab.wait_select_single(
            'QQuickItem', focus=True)
        today_temp = today_item.wait_select_single(
            'QQuickText', objectName='CurrentTempText')
        today_temp_scale = today_item.select_single(
            'QQuickText', objectName='CurrentTempScale')
        tomorrow_item = today_item.get_parent().get_children_by_type(
            'QQuickItem', focus=False, z=1)
        tomorrow_temp = tomorrow_item[0].select_single(
            'QQuickText', objectName='CurrentTempText')
        if units == "imperial":
            self.assertThat(today_temp_scale.text, Eventually(Equals(u'°F')))
            self.assertThat(today_temp.text, Eventually(Equals(u'46')))
            self.assertThat(tomorrow_temp.text, Eventually(Equals(u'48')))
        else:
            self.assertThat(today_temp_scale.text, Eventually(Equals(u'°C')))
            self.assertThat(today_temp.text, Eventually(Equals(u'8')))
            self.assertThat(tomorrow_temp.text, Eventually(Equals(u'9')))

    def _check_wind_units(self, wind_units):
        """Checks selected units by values from the first location tab"""
        if self.main_view.wideAspect:
            self._ensure_tablet_tabs_loaded()
        self.assertThat(lambda: self.main_view.select_many(
            'WeatherDetailComponent', objectName='WindSpeedValue')[0],
            Eventually(Not(Is(None))))
        current_location_tab = self.main_view.select_single(
            'LocationTab', visible=True)
        focused_item = current_location_tab.select_single(
            'QQuickItem', focus=True)
        wind_component = focused_item.select_single(
            'WeatherDetailComponent', objectName='WindSpeedValue')
        self.assertThat(lambda: wind_component.select_single(
            'Label', objectName='WeatherDetailUnit'),
            Eventually(Not(Is(None))))
        wind_unit = wind_component.select_single(
            'Label', objectName='WeatherDetailUnit')
        self.assertThat(lambda: wind_component.select_single(
            'QQuickText', objectName='WeatherDetailValue'),
            Eventually(Not(Is(None))))
        wind_value = wind_component.select_single(
            'QQuickText', objectName='WeatherDetailValue')
        if wind_units == "mph":
            self.assertThat(wind_unit.text, Eventually(Equals(u'mph W')))
            self.assertThat(wind_value.text, Eventually(Equals(u'19')))
        else:
            self.assertThat(wind_unit.text, Eventually(Equals(u'km/h W')))
            self.assertThat(wind_value.text, Eventually(Equals(u'32')))

    def _check_precipitation_units(self, pre_units):
        """Checks selected units by values from the first location tab"""
        if self.main_view.wideAspect:
            self._ensure_tablet_tabs_loaded()
        self.assertThat(lambda: self.main_view.select_many(
            'WeatherDetailComponent', objectName='PrecipitationValue')[0],
            Eventually(Not(Is(None))))
        current_location_tab = self.main_view.select_single(
            'LocationTab', visible=True)
        focused_item = current_location_tab.select_single(
            'QQuickItem', focus=True)
        component = focused_item.select_single(
            'WeatherDetailComponent', objectName='PrecipitationValue')
        self.assertThat(lambda: component.select_many(
            'QQuickText', objectName='WeatherDetailUnit'),
            Eventually(Not(Is(None))))
        unit = component.select_single('QQuickText',
                                       objectName='WeatherDetailUnit')
        self.assertThat(lambda: component.select_single(
            'QQuickText', objectName='WeatherDetailValue'),
            Eventually(Not(Is(None))))
        value = component.select_single('QQuickText',
                                        objectName='WeatherDetailValue')
        if pre_units == "in":
            self.assertThat(unit.text, Eventually(Equals(u'in')))
            self.assertThat(value.text, Eventually(Equals(u'0.18')))
        else:
            self.assertThat(unit.text, Eventually(Equals(u'mm')))
            self.assertThat(value.text, Eventually(Equals(u'0.5')))

    def _check_service(self, service):
        """
        Checks selected units weather data service from the first location tab
        """
        if self.main_view.wideAspect:
            self._ensure_tablet_tabs_loaded()
        self.assertThat(lambda: self.main_view.select_many(
            'TabFooter', objectName='TabFooter')[0],
            Eventually(Not(Is(None))))
        current_location_tab = self.main_view.select_single('LocationTab',
                                                            visible=True)
        self.assertThat(lambda: current_location_tab.select_single(
            'TabFooter', objectName='TabFooter'),
            Eventually(Not(Is(None))))
        self.assertThat(lambda: current_location_tab.select_single(
            'QQuickImage', objectName='DataProviderLogo'),
            Eventually(Not(Is(None))))

        tab_footer = current_location_tab.select_single('TabFooter',
                                                        objectName='TabFooter')
        provider_logo = current_location_tab.select_single(
            'QQuickImage', objectName='DataProviderLogo')
        if service == "weatherchannel":
            self.assertThat(tab_footer.visible, Eventually(Equals(True)))
            self.assertThat(provider_logo.visible, Eventually(Equals(False)))
        elif service == "openweathermap":
            self.assertThat(tab_footer.visible, Eventually(Equals(False)))
            self.assertThat(provider_logo.visible, Eventually(Equals(True)))

    def test_switch_scale(self):
        """Tests switching the scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_units('metric')
        self._open_settings_sheet()

        # open the value selector
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='TemperatureUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="TemperatureUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, fahrenheit
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        imperial_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(imperial_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self._click_sheet_confirm()

        # wait for reload and check the imperial values
        self.main_view.wait_for_activity_to_finish()
        self._check_units('imperial')

        # switch back to metric values again
        self._open_settings_sheet()
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='TemperatureUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="TemperatureUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(current_option)
        # while it's opened, units_selector returns false
        # self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click celsius option
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        metric_option = units_selector.select_single(
            'OptionSelectorDelegate', objectName='celsiusOption')
        self.pointing_device.click_object(metric_option)

        # confirm
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self._click_sheet_confirm()

        # wait for reload and check the metric values again
        self.main_view.wait_for_activity_to_finish()
        self._check_units('metric')

    def test_switch_scale_cancel(self):
        """Test canceling switching scale"""
        # first check metric values and open the settings sheet
        self._check_units('metric')
        self._check_wind_units('kmh')
        self._open_settings_sheet()

        # open the temp value selector
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='TemperatureUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="TemperatureUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, fahrenheit
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        imperial_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(imperial_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))

        # open the wind value selector
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='WindUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="WindUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.pointing_device.click_object(units_selector)

        # choose second option, mph
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        mph_option = units_selector.select_many('OptionSelectorDelegate')[1]
        self.pointing_device.click_object(mph_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))

        # cancel and check
        self._click_sheet_cancel()
        self._check_units('metric')
        self._check_wind_units('kmh')

    def test_switch_wind_scale(self):
        """Tests switching the wind scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_wind_units('kmh')
        self._open_settings_sheet()

        # open the wind value selector
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='WindUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="WindUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, mph
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        mph_option = units_selector.select_many('OptionSelectorDelegate')[0]
        self.pointing_device.click_object(mph_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self._click_sheet_confirm()

        # wait for reload and check the mph values
        self.main_view.wait_for_activity_to_finish()
        self._check_wind_units('mph')

        # switch back to kmh values again
        self._open_settings_sheet()
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='WindUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="WindUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(current_option)
        # while it's opened, units_selector returns false
        # self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click kmh option
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        kmh_option = units_selector.select_single(
            'OptionSelectorDelegate', objectName='kmhOption')
        self.pointing_device.click_object(kmh_option)

        # confirm
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self._click_sheet_confirm()

        # wait for reload and check the metric values again
        self.main_view.wait_for_activity_to_finish()
        self._check_wind_units('kmh')

    @skip("data not available")
    def test_switch_precipitation_scale(self):
        """Tests switching the precipition scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_precipitation_units('mm')
        self._open_settings_sheet()

        # open the precipitation value selector
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='PrecipitationUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="PrecipitationUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, inch
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        in_option = units_selector.select_many('OptionSelectorDelegate')[0]
        self.pointing_device.click_object(in_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self._click_sheet_confirm()

        # wait for reload and check the inch values
        self.main_view.wait_for_activity_to_finish()
        self._check_wind_units('in')

        # switch back to mm values again
        self._open_settings_sheet()
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='PrecipitationUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="PrecipitationUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(current_option)
        # while it's opened, units_selector returns false
        # self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click mm option
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        # XXX select the option by it's object name or index, using the
        # emulator from the ubuntu-ui-toolkit.
        mm_option = units_selector.select_single(
            'OptionSelectorDelegate', objectName='millimetersOption')
        self.pointing_device.click_object(mm_option)

        # confirm
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self._click_sheet_confirm()

        # wait for reload and check the metric values again
        self.main_view.wait_for_activity_to_finish()
        self._check_precipitation_units('mm')

    def test_switch_service(self):
        """Tests switching the scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_service('weatherchannel')
        self._open_settings_sheet()

        # open the value selector
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='ServiceSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="ServiceSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, openweathermap
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        second_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(second_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))

        # check if precipitation units selector appears
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='PrecipitationUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="PrecipitationUnitsSelector")
        self.assertThat(units_selector.visible, Eventually(Equals(1)))
        self._click_sheet_confirm()

        # wait for reload and check the used service
        self.main_view.wait_for_activity_to_finish()
        self._check_service('openweathermap')

        # switch back to metric values again
        self._open_settings_sheet()
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='ServiceSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="ServiceSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[0], Eventually(Not(Is(None))))
        current_option = units_selector.select_many(
            'OptionSelectorDelegate')[0]
        self.pointing_device.click_object(current_option)
        # while it's opened, units_selector returns false
        # self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click twc option
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many(
            'OptionSelectorDelegate')[1], Eventually(Not(Is(None))))
        first_option = units_selector.select_single(
            'OptionSelectorDelegate', objectName='twcOption')
        self.pointing_device.click_object(first_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))

        # check if precipitation units selector disappears and confirm
        self.assertThat(lambda: self.main_view.select_single(
            'OptionSelector', objectName='PrecipitationUnitsSelector'),
            Eventually(Not(Is(None))))
        units_selector = self.main_view.select_single(
            'OptionSelector', objectName="PrecipitationUnitsSelector")
        self.assertThat(units_selector.visible, Eventually(Equals(0)))
        self._click_sheet_confirm()

        # wait for reload and check the metric values again
        self.main_view.wait_for_activity_to_finish()
        self._check_service('weatherchannel')
